#include "bon_led_ctrl.h"

static int lnb_powered = 0;

#define FRIIO_PIC_LNB_0		(0)
#define FRIIO_PIC_STROBE_0	(0)
#define FRIIO_PIC_CLK_0		(0)
#define FRIIO_PIC_LED_0		(0)

#define FRIIO_PIC_LNB_1		(1 << 0)
#define FRIIO_PIC_STROBE_1	(1 << 1)
#define FRIIO_PIC_CLK_1		(1 << 2)
#define FRIIO_PIC_LED_1		(1 << 3)

/*
	bit 0   LNB 
	bit 1   STROBE
	bit 2   CLK
	bit 3   LED control
*/

static void UsbSendLED_bit(int fd, unsigned char data)
{
	UsbSendVendorRequest(fd, data | FRIIO_PIC_CLK_0, 0x0000);
	UsbSendVendorRequest(fd, data | FRIIO_PIC_CLK_1, 0x0000);
}

static void UsbSendLED(int fd, unsigned char *data, int len)
{
	int i, j;
	int lnb = lnb_powered ? FRIIO_PIC_LNB_1 : FRIIO_PIC_LNB_0;

	UsbSendLED_bit(fd, FRIIO_PIC_LED_1 | lnb | FRIIO_PIC_STROBE_1);
	UsbSendLED_bit(fd, FRIIO_PIC_LED_0 | lnb | FRIIO_PIC_STROBE_1);

	for (i = 0; i < len; i++) {
		for (j = 0; j < 8; j++) {
			int led_bit = (data[i] & (0x80U >> j)) ? FRIIO_PIC_LED_1 : FRIIO_PIC_LED_0;

			UsbSendLED_bit(fd, led_bit | lnb | FRIIO_PIC_STROBE_1);
		}
	}

	// STROBE: OFF
	UsbSendLED_bit(fd, FRIIO_PIC_LED_0 | lnb | FRIIO_PIC_STROBE_0);
}

void UsbProcLED_full(int fd, int rgb_saturation, uint32_t color)
{
	unsigned char data[4];

	data[0] = rgb_saturation;
	data[1] = (color >> 16) & 0xff;
	data[2] = (color >> 8) & 0xff;
	data[3] = (color >> 0) & 0xff;
	UsbSendLED(fd, data, sizeof(data));
}

void UsbProcLED(int fd, uint8_t bType)
{
	switch(bType)
	{
	case BON_LED_RED:
		UsbProcLED_full(fd, 100, 0xFF000A);
		break;
	case BON_LED_GREEN:
		UsbProcLED_full(fd, 100, 0x00FF64);
		break;
	case BON_LED_PURPLE:
		UsbProcLED_full(fd, 150, 0xFF00FF);
		break;
	case BON_LED_WHITE:
		UsbProcLED_full(fd, 100, 0xFFFFFF);
		break;
	}
}

void SetLNBPowered(int powered)
{
	lnb_powered = powered;
}
